#ifndef LIBUCDAEMON_H
#define LIBUCDAEMON_H

#include <signal.h>
#include <stdarg.h>
#include <stdint.h>
#include <sys/types.h>

/* Daemonizing */

#define UCD_NOCHDIR		1
#define UCD_NOCLOSE		2
#define UCD_NODETACH		4

int ucd_daemon_start(char *name, int flags,
		     int (*parent_init)(int argc, char *argv[]),
		     int (*daemon_main)(int argc, char *argv[]),
		     int argc, char *argv[]);

void ucd_daemon_wait(unsigned int timeout);
void ucd_daemon_signal();
void ucd_daemon_signal_status(int status);

int ucd_daemon_is_detached(void);

/* Process name */

const char *ucd_get_name(void);
void ucd_set_name(char *name);

/* Logging */

void ucd_log(int priority, const char *format, ...);
void ucd_vlog(int priority, const char *format, va_list ap);
void ucd_error(const char *format, ...);

/* PID file */

pid_t ucd_pidfile_read(const char *path);
void ucd_pidfile_lock(const char *path);
void ucd_pidfile_unlock();

/* signal handling */

typedef struct ucd_sigpipe_t ucd_sigpipe_t;

ucd_sigpipe_t *ucd_sigpipe_open(sigset_t *set);
int ucd_sigpipe_close(ucd_sigpipe_t *sigpipe);
int ucd_sigpipe_get_fd(ucd_sigpipe_t *sigpipe);
int ucd_sigpipe_get_signal(ucd_sigpipe_t *sigpipe, int *pcount);

/* file descriptor readiness poll */

#define UCD_POLL_READ		1
#define UCD_POLL_WRITE  	2
#define UCD_POLL_ONESHOT 	4

typedef intptr_t ucd_poll_t;
typedef intptr_t ucd_poll_fd_t;

typedef void (*ucd_fd_handler_t)(int fd, int flags, intptr_t data);

ucd_poll_t ucd_poll_open(int size);
int ucd_poll_close(ucd_poll_t pd);
ucd_poll_fd_t ucd_poll_add_fd(ucd_poll_t pd, int fd, int flags, ucd_fd_handler_t cb, intptr_t cbdata);
int ucd_poll_modify_fd(ucd_poll_t pd, ucd_poll_fd_t pfd, int flags);
int ucd_poll_delete_fd(ucd_poll_t pd, ucd_poll_fd_t pfd);
int ucd_poll_dispatch(ucd_poll_t pd, int timeout);

#endif
